# coding=utf-8
"""Complete set of THERM Simulation Settings."""
from __future__ import division

from .mesh import MeshControl


class SimulationParameter(object):
    """Complete set of Therm Simulation Settings.

    Args:
        mesh: A MeshControl that lists the desired meshing procedure. If None,
            default meshing control will be automatically generated. (Default: None).

    Properties:
        * mesh
    """
    __slots__ = ('_mesh',)

    def __init__(self, mesh=None):
        """Initialize SimulationParameter."""
        self.mesh = mesh

    @property
    def mesh(self):
        """Get or set a MeshControl object for the simulation meshing procedure."""
        return self._mesh

    @mesh.setter
    def mesh(self, value):
        if value is not None:
            assert isinstance(value, MeshControl), 'Expected MeshControl ' \
                'for SimulationParameter output. Got {}.'.format(type(value))
            self._mesh = value
        else:
            self._mesh = MeshControl()

    @classmethod
    def from_dict(cls, data):
        """Create a SimulationParameter object from a dictionary.

        Args:
            data: A SimulationParameter dictionary in following the format below.

        .. code-block:: python

            {
            "type": "SimulationParameter",
            "mesh": {} # Fairyfly MeshControl dictionary
            }
        """
        assert data['type'] == 'SimulationParameter', \
            'Expected SimulationParameter dictionary. Got {}.'.format(data['type'])
        mesh = None
        if 'mesh' in data and data['mesh'] is not None:
            mesh = MeshControl.from_dict(data['mesh'])
        return cls(mesh)

    def to_dict(self):
        """SimulationParameter dictionary representation."""
        return {
            'type': 'SimulationParameter',
            'mesh': self.mesh.to_dict()
        }

    def duplicate(self):
        """Get a copy of this object."""
        return self.__copy__()

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __copy__(self):
        return SimulationParameter(self.mesh.duplicate())

    def __key(self):
        """A tuple based on the object properties, useful for hashing."""
        return (hash(self.mesh),)

    def __hash__(self):
        return hash(self.__key())

    def __eq__(self, other):
        return isinstance(other, SimulationParameter) and self.__key() == other.__key()

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return 'Therm SimulationParameter:'



"""Commands to set fairyfly-therm configurations."""
import click
import sys
import logging
import json

from fairyfly_therm.config import folders

_logger = logging.getLogger(__name__)


@click.group(help='Commands to set fairyfly-therm configurations.')
def set_config():
    pass


@set_config.command('therm-path')
@click.argument('folder-path', required=False, type=click.Path(
    exists=True, file_okay=False, dir_okay=True, resolve_path=True))
def therm_path(folder_path):
    """Set the therm-path configuration variable.

    \b
    Args:
        folder_path: Path to a folder to be set as the therm-path.
            If unspecified, the therm-path will be set back to the default.
    """
    _set_config_variable(folder_path, 'therm_path')


@set_config.command('lbnl-data-path')
@click.argument('folder-path', required=False, type=click.Path(
    exists=True, file_okay=False, dir_okay=True, resolve_path=True))
def lbnl_data_path(folder_path):
    """Set the lbnl-data-path configuration variable.

    \b
    Args:
        folder_path: Path to a folder to be set as the lbnl-data-path.
            If unspecified, the lbnl-data-path will be set back to the default.
    """
    _set_config_variable(folder_path, 'lbnl_data_path')


@set_config.command('therm-lib-path')
@click.argument('folder-path', required=False, type=click.Path(
    exists=True, file_okay=False, dir_okay=True, resolve_path=True))
def therm_lib_path(folder_path):
    """Set the therm-lib-path configuration variable.

    \b
    Args:
        folder_path: Path to a folder to be set as the therm-lib-path.
            If unspecified, the therm-lib-path will be set back to the default.
    """
    _set_config_variable(folder_path, 'therm_lib_path')


def _set_config_variable(folder_path, variable_name):
    var_cli_name = variable_name.replace('_', '-')
    try:
        config_file = folders.config_file
        with open(config_file) as inf:
            data = json.load(inf)
        data[variable_name] = folder_path if folder_path is not None else ''
        with open(config_file, 'w') as fp:
            json.dump(data, fp, indent=4)
        msg_end = 'reset to default' if folder_path is None \
            else 'set to: {}'.format(folder_path)
        print('{} successfully {}.'.format(var_cli_name, msg_end))
    except Exception as e:
        _logger.exception('Failed to set {}.\n{}'.format(var_cli_name, e))
        sys.exit(1)
    else:
        sys.exit(0)
